@@ -20,30 +20,41 @@
   nXblocks <- length(Xlist)
   
   Yvariables    <- colnames(Y)
-  #nYvariables   <- length(Yvariables)
-  
+
   inertieY <- sum(diag(t(as.matrix(scale(Y, scale=FALSE)))%*%(as.matrix(scale(Y, scale=FALSE)))))
 
-  # datasets
+  # partition
   set.seed(seed = seed)
-  
-  obsList     <- list()
-  initialYrep <- list()
-  Yrep        <- list()
-  Xlistrep    <- list()
-  
-  initialY     <- Y
-  
-  if(nbrep==1){
-    initialYrep[[1]] <- initialY
-    Yrep[[1]]        <- Y
-    Xlistrep[[1]]    <- Xlist
-  }else{
-    for(i in 1:nbrep){
-      obsList[[i]]      <- sample(x=n, size=n, replace = FALSE, prob = NULL)
-      initialYrep[[i]]  <- initialY[obsList[[i]],, drop=FALSE]
-      Yrep[[i]]         <- Y[obsList[[i]],, drop=FALSE]
-      Xlistrep[[i]]     <- lapply(1:length(Xlist), function (j) Xlist[[j]][obsList[[i]],,drop=FALSE])
+  if(cvmethod=="loo"){CVtype<- segmkf(n = nrow(Xlist[[1]]), K = nrow(Xlist[[1]]), type = "random", nrep = 1)}#=as.list(1:n)}
+  if((cvmethod=="kfolds") & (is.null(samplingk)==TRUE)){
+    CVtype <- segmkf(n = nrow(Xlist[[1]]), K = nfolds, type = "random", nrep = nbrep)
+    for(r in 1:nbrep){
+      for(ss in 1:nfolds){
+        CVtype[[r]][[ss]] <- sort(CVtype[[r]][[ss]])
+      }
+    }
+  }
+  if((cvmethod=="kfolds") & (is.null(samplingk)==FALSE)){
+    samplingktab <- table(samplingk)
+    partCVtype <- list()
+    for(s in 1:length(names(samplingktab))){
+      partCVtype[[s]] <- segmkf(n = samplingktab[s], K = nfolds, type = "random", nrep = nbrep)
+      for(r in 1:nbrep){
+        for(ss in 1:length(partCVtype[[s]][[r]])){
+          partCVtype[[s]][[r]][[ss]] <- which(samplingk==names(samplingktab)[s])[partCVtype[[s]][[r]][[ss]]]
+        }
+      }
+    }
+    CVtype <- list()
+    for(r in 1:nbrep){
+      CVtype[[r]] <- list()
+      for(ss in 1:nfolds){
+        CVtype[[r]][[ss]] <- c(0)
+        for(s in 1:length(names(samplingktab))){
+          CVtype[[r]][[ss]] <- c(CVtype[[r]][[ss]], partCVtype[[s]][[r]][[ss]])
+        }
+        CVtype[[r]][[ss]] <- sort(CVtype[[r]][[ss]][-1])
+      }
     }
   }
 
@@ -70,46 +81,22 @@
   	
 
     for(i in 1:nbrep){
-      
-      # partition
-      set.seed(seed = seed)
-      if(cvmethod=="loo"){CVtype=as.list(1:n)}
-      if((cvmethod=="kfolds") & (is.null(samplingk)==TRUE)){
-        CVtype <- segmkf(n = n, K = nfolds, type = "interleaved")$rep1
-      }
-      if((cvmethod=="kfolds") & (is.null(samplingk)==FALSE)){
-        samplingktab <- table(samplingk)
-        for(s in 1:length(names(samplingktab))){
-          partCVtype[[s]] <- segmkf(n = samplingktab[s], K = nfolds, type = "interleaved")$rep1
-          for(ss in 1:length(partCVtype[[s]])){
-            partCVtype[[s]][[ss]] <- which(samplingk==names(samplingktab)[s])[partCVtype[[s]][[ss]]]
-          }
-        }
-        CVtype <- list()
-        for(sss in 1:nfolds){
-          CVtype[[sss]] <- partCVtype[[1]][[sss]]
-          for(s in 2:length(names(samplingktab))){
-            CVtype[[sss]] <- c(CVtype[[sss]],partCVtype[[s]][[sss]])
-          }
-        }
-      }
-      
-      
+
       for(j in 1:nrow(lvcombi)){
         reppredCV <- matrix(NA, nrow=n, ncol = length(Yvariables))
-        for(k in 1:length(CVtype)){
-          Xlisttrain <-lapply(1:length(Xlistrep[[i]]),function(x) Xlistrep[[i]][[x]][-CVtype[[k]],,drop=FALSE])
-          Ytrain <- Yrep[[i]][-CVtype[[k]],,drop=FALSE]
-          Xlisttest <- lapply(1:length(Xlistrep[[i]]),function(x) Xlistrep[[i]][[x]][CVtype[[k]],,drop=FALSE])
-          Ytest <- Yrep[[i]][CVtype[[k]],,drop=FALSE]
-          repmodel <- soplsr(Xlist=Xlisttrain, Y=Ytrain, Xscaling = Xscaling, Yscaling = Yscaling, weights = weights, nlv=lvcombi[j,])
-          reppredCV[CVtype[[k]],] <- predict(repmodel, Xlisttest)
+        for(k in 1:length(CVtype[[i]])){
+          Xlisttrain <- lapply(1:length(Xlist),function(x) Xlist[[x]][-CVtype[[i]][[k]],,drop=FALSE])
+          Ytrain <- Y[-CVtype[[i]][[k]],,drop=FALSE]
+          Xlisttest <- lapply(1:length(Xlist),function(x) Xlist[[x]][CVtype[[i]][[k]],,drop=FALSE])
+          Ytest <- Y[CVtype[[i]][[k]],,drop=FALSE]
+          repmodel <- soplsr(Xlist=Xlisttrain, Y=Ytrain, Xscaling = Xscaling, Yscaling = Yscaling, weights = weights[-CVtype[[i]][[k]]], nlv=lvcombi[j,])
+          reppredCV[CVtype[[i]][[k]],] <- predict(repmodel, Xlisttest)[order(CVtype[[i]][[k]]),]
         }
-        SqErrCV <- (Yrep[[i]]-reppredCV)^2
+        SqErrCV <- (Y-reppredCV)^2
         
         Rep_rmseCV_byY[j,i,]    <- sqrt(apply(SqErrCV,2,mean))
-        Rep_ExplVarCV_byY[j,i,] <- 1- ((Rep_rmseCV_byY[j,i,]^2)/matrix((apply(Yrep[[i]],2,var)*(n-1)/n), ncol=ncol(Yrep[[i]])))
-        Rep_rmseCV[j,i,]    <- sqrt(apply(Rep_rmseCV_byY[j,i,,drop=FALSE]^2, FUN = mean, MARGIN = 1))
+        Rep_ExplVarCV_byY[j,i,] <- r2(reppredCV,Y)# 1- ((Rep_rmseCV_byY[j,i,]^2)/matrix((apply(Y,2,var)*(n-1)/n), ncol=ncol(Y)))
+        Rep_rmseCV[j,i,]    <- sqrt(mean(SqErrCV))
         Rep_ExplVarCV[j,i,] <- apply(Rep_ExplVarCV_byY[j,i,,drop=FALSE], FUN = mean, MARGIN = 1)
       }
     }
@@ -149,7 +136,7 @@
         if(selection=="localmin"){
           if(nrow(res_nlvsum_rmseCV_Ysel)>1){
             # sign of the difference of accuracies to select the optim combination with the lower total number of components
-            rtsdiff <- c(NA,sapply(2:nrow(res_nlvsum_rmseCV_Ysel), function(i)((res_nlvsum_rmseCV_Ysel$mean[i]-res_nlvsum_rmseCV_Ysel$mean[i-1])<=0)))
+            rtsdiff <- c(NA,sapply(2:nrow(res_nlvsum_rmseCV_Ysel), function(i)((res_nlvsum_rmseCV_Ysel$mean[i]-res_nlvsum_rmseCV_Ysel$mean[i-1])<0)))
             choiceYH[yy] <- min(res_nlvsum_rmseCV_Ysel[(which(rtsdiff==FALSE)-1)[1],"index"],res_nlvsum_rmseCV_Ysel[nrow(res_nlvsum_rmseCV_Ysel),"index"],na.rm=TRUE)
           }else{
             choiceYH[yy]<- 1
@@ -163,10 +150,10 @@
             # one standard error rule to select the optim number of components
             minmean    <- which.min(res_nlvsum_rmseCV_Ysel$mean)[1]
             threshmean <- res_nlvsum_rmseCV_Ysel$mean[minmean] + res_nlvsum_rmseCV_Ysel$sd[minmean]
-            if((minmean == 1) | (sum(res_nlvsum_rmseCV_Ysel$mean[1:minmean]>=threshmean)==0)){
+            if((minmean == 1) | (sum(res_nlvsum_rmseCV_Ysel$mean[1:minmean]<=threshmean)==0)){
               choiceYH[yy] <- 1
             }else{
-              choiceYH[yy] <- max(res_nlvsum_rmseCV_Ysel[which(res_nlvsum_rmseCV_Ysel$mean[1:minmean]>=threshmean),"index"])
+              choiceYH[yy] <- min(res_nlvsum_rmseCV_Ysel[which(res_nlvsum_rmseCV_Ysel$mean[1:minmean]<=threshmean),"index"])
             }
           }else{
             choiceYH[yy] <- 1
@@ -192,7 +179,7 @@
       if(selection=="localmin"){
         if(nrow(res_nlvsum_rmseCV_Ysel)>1){
           # sign of the difference of accuracies to select the optim combination with the lower total number of components
-          rtsdiff <- c(NA,sapply(2:nrow(res_nlvsum_rmseCV_Ysel), function(i)((res_nlvsum_rmseCV_Ysel$mean[i]-res_nlvsum_rmseCV_Ysel$mean[i-1])<=0)))
+          rtsdiff <- c(NA,sapply(2:nrow(res_nlvsum_rmseCV_Ysel), function(i)((res_nlvsum_rmseCV_Ysel$mean[i]-res_nlvsum_rmseCV_Ysel$mean[i-1])<0)))
           kchoix <- min(res_nlvsum_rmseCV_Ysel[(which(rtsdiff==FALSE)-1)[1],"index"],res_nlvsum_rmseCV_Ysel[nrow(res_nlvsum_rmseCV_Ysel),"index"],na.rm=TRUE)
         }else{
           kchoix <- 1
@@ -206,10 +193,10 @@
           # one standard error rule to select the optim number of components
           minmean    <- which.min(res_nlvsum_rmseCV_Ysel$mean)[1]
           threshmean <- res_nlvsum_rmseCV_Ysel$mean[minmean] + res_nlvsum_rmseCV_Ysel$sd[minmean]
-          if((minmean == 1) | (sum(res_nlvsum_rmseCV_Ysel$mean[1:minmean]>=threshmean)==0)){
+          if((minmean == 1) | (sum(res_nlvsum_rmseCV_Ysel$mean[1:minmean]<=threshmean)==0)){
             kchoix <- 1
           }else{
-            kchoix <- max(res_nlvsum_rmseCV_Ysel[which(res_nlvsum_rmseCV_Ysel$mean[1:minmean]>=threshmean),"index"])
+            kchoix <- min(res_nlvsum_rmseCV_Ysel[which(res_nlvsum_rmseCV_Ysel$mean[1:minmean]<=threshmean),"index"])
           }
         }else{
           kchoix <- 1
@@ -220,9 +207,7 @@
 
     # outputs
     rts <- list(lvcombi=lvcombi, 
-                #optimCombiLine=kchoix, 
                 optimcombi=unlist(lvcombi[kchoix,,drop=FALSE]), 
-                #optimExplVarCV=res_ExplVarCV[kchoix,,drop=FALSE], 
                 rmseCV_byY=res_rmseCV_byY, 
                 ExplVarCV_byY=res_ExplVarCV_byY, 
                 rmseCV=res_rmseCV, 
@@ -243,31 +228,6 @@
     Rep_rmseCV_byY <- Rep_ExplVarCV_byY <- list()
     Rep_rmseCV     <- Rep_ExplVarCV     <- list()
     
-    for(i in 1:nbrep){
-      # partition
-      set.seed(seed = seed)
-      if(cvmethod=="loo"){CVtype=as.list(1:n)}
-      if((cvmethod=="kfolds") & (is.null(samplingk)==TRUE)){
-        CVtype <- segmkf(n = n, K = nfolds, type = "interleaved")$rep1
-      }
-      if((cvmethod=="kfolds") & (is.null(samplingk)==FALSE)){
-        samplingktab <- table(samplingk)
-        for(s in 1:length(names(samplingktab))){
-          partCVtype[[s]] <- segmkf(n = samplingktab[s], K = nfolds, type = "interleaved")$rep1
-          for(ss in 1:length(partCVtype[[s]])){
-            partCVtype[[s]][[ss]] <- which(samplingk==names(samplingktab)[s])[partCVtype[[s]][[ss]]]
-          }
-        }
-        CVtype <- list()
-        for(sss in 1:nfolds){
-          CVtype[[sss]] <- partCVtype[[1]][[sss]]
-          for(s in 2:length(names(samplingktab))){
-            CVtype[[sss]] <- c(CVtype[[sss]],partCVtype[[s]][[sss]])
-          }
-        }
-      }
-    }
-    
     for (m in 1:nXblocks) {
       # combinations
       if(m==1){
@@ -295,20 +255,20 @@
       for(i in 1:nbrep){
         for(j in 1:nrow(lvcombi[[m]])){
           reppredCV <- matrix(NA, nrow=n, ncol = length(Yvariables))
-          for(k in 1:length(CVtype)){
-            Xlisttrain <-lapply(1:m,function(x) Xlistrep[[i]][[x]][-CVtype[[k]],,drop=FALSE])
-            Ytrain <- Yrep[[i]][-CVtype[[k]],,drop=FALSE]
-            Xlisttest <- lapply(1:m,function(x) Xlistrep[[i]][[x]][CVtype[[k]],,drop=FALSE])
-            Ytest <- Yrep[[i]][CVtype[[k]],,drop=FALSE]
-            repmodel <- soplsr(Xlist=Xlisttrain, Y=Ytrain, Xscaling = Xscaling, Yscaling = Yscaling, weights = weights, nlv=unlist(lvcombi[[m]][j,]))
-            reppredCV[CVtype[[k]],] <- predict(repmodel, Xlisttest)
+          for(k in 1:length(CVtype[[i]])){
+            Xlisttrain <-lapply(1:m,function(x) Xlist[[x]][-CVtype[[i]][[k]],,drop=FALSE])
+            Ytrain <- Y[-CVtype[[i]][[k]],,drop=FALSE]
+            Xlisttest <- lapply(1:m,function(x) Xlist[[x]][CVtype[[i]][[k]],,drop=FALSE])
+            Ytest <- Y[CVtype[[i]][[k]],,drop=FALSE]
+            repmodel <- soplsr(Xlist=Xlisttrain, Y=Ytrain, Xscaling = Xscaling, Yscaling = Yscaling, weights = weights[-CVtype[[i]][[k]]], nlv=unlist(lvcombi[[m]][j,]))
+            reppredCV[CVtype[[i]][[k]],] <- predict(repmodel, Xlisttest)[order(CVtype[[i]][[k]]),]
           }
-          SqErrCV <- (Yrep[[i]]-reppredCV)^2
+          SqErrCV <- (Y-reppredCV)^2
 
           Rep_rmseCV_byY[[m]][j,i,]    <- sqrt(apply(SqErrCV,2,mean))
-          Rep_ExplVarCV_byY[[m]][j,i,] <- 1- ((Rep_rmseCV_byY[[m]][j,i,]^2)/matrix((apply(Yrep[[i]],2,var)*(n-1)/n), ncol=ncol(Yrep[[i]])))
+          Rep_ExplVarCV_byY[[m]][j,i,] <- r2(reppredCV,Y)# 1- ((Rep_rmseCV_byY[[m]][j,i,]^2)/matrix((apply(Y,2,var)*(n-1)/n), ncol=ncol(Y)))
 
-          Rep_rmseCV[[m]][j,i,]    <- sqrt(apply(Rep_rmseCV_byY[[m]][j,i,,drop=FALSE]^2, FUN = mean, MARGIN = 1))
+          Rep_rmseCV[[m]][j,i,]    <- sqrt(mean(SqErrCV))
           Rep_ExplVarCV[[m]][j,i,] <- apply(Rep_ExplVarCV_byY[[m]][j,i,,drop=FALSE], FUN = mean, MARGIN = 1)
         }
       }
@@ -357,7 +317,7 @@
           
           if(selection=="localmin"){
             if(nrow(res_nlvsum_rmseCV_Ysel)>1){
-              rtsdiff <- c(NA,sapply(2:nrow(res_nlvsum_rmseCV_Ysel), function(i)((res_nlvsum_rmseCV_Ysel$mean[i]-res_nlvsum_rmseCV_Ysel$mean[i-1])<=0)))
+              rtsdiff <- c(NA,sapply(2:nrow(res_nlvsum_rmseCV_Ysel), function(i)((res_nlvsum_rmseCV_Ysel$mean[i]-res_nlvsum_rmseCV_Ysel$mean[i-1])<0)))
               choiceYH[yy] <- min(res_nlvsum_rmseCV_Ysel[(which(rtsdiff==FALSE)-1)[1],"index"],res_nlvsum_rmseCV_Ysel[nrow(res_nlvsum_rmseCV_Ysel),"index"],na.rm=TRUE)
             }else{
               choiceYH[yy]<- 1
@@ -371,10 +331,10 @@
               # one standard error rule
               minmean    <- which.min(res_nlvsum_rmseCV_Ysel$mean)[1]
               threshmean <- res_nlvsum_rmseCV_Ysel$mean[minmean] + res_nlvsum_rmseCV_Ysel$sd[minmean]
-              if((minmean == 1) | (sum(res_nlvsum_rmseCV_Ysel$mean[1:minmean]>=threshmean)==0)){
+              if((minmean == 1) | (sum(res_nlvsum_rmseCV_Ysel$mean[1:minmean]<=threshmean)==0)){
                 choiceYH[yy] <- 1
               }else{
-                choiceYH[yy] <- max(res_nlvsum_rmseCV_Ysel[which(res_nlvsum_rmseCV_Ysel$mean[1:minmean]>=threshmean),"index"])
+                choiceYH[yy] <- min(res_nlvsum_rmseCV_Ysel[which(res_nlvsum_rmseCV_Ysel$mean[1:minmean]<=threshmean),"index"])
               }
             }else{
               choiceYH[yy] <- 1
@@ -410,7 +370,7 @@
         
         if(selection=="localmin"){
           if(nrow(res_nlvsum_rmseCV_Ysel)>1){
-            rtsdiff <- c(NA,sapply(2:nrow(res_nlvsum_rmseCV_Ysel), function(i)((res_nlvsum_rmseCV_Ysel$mean[i]-res_nlvsum_rmseCV_Ysel$mean[i-1])<=0)))
+            rtsdiff <- c(NA,sapply(2:nrow(res_nlvsum_rmseCV_Ysel), function(i)((res_nlvsum_rmseCV_Ysel$mean[i]-res_nlvsum_rmseCV_Ysel$mean[i-1])<0)))
             kchoix <- min(res_nlvsum_rmseCV_Ysel[(which(rtsdiff==FALSE)-1)[1],"index"],res_nlvsum_rmseCV_Ysel[nrow(res_nlvsum_rmseCV_Ysel),"index"],na.rm=TRUE)
           }else{
             kchoix <- 1
@@ -424,10 +384,10 @@
             # one standard error rule 
             minmean    <- which.min(res_nlvsum_rmseCV_Ysel$mean)[1]
             threshmean <- res_nlvsum_rmseCV_Ysel$mean[minmean] + res_nlvsum_rmseCV_Ysel$sd[minmean]
-            if((minmean == 1) | (sum(res_nlvsum_rmseCV_Ysel$mean[1:minmean]>=threshmean)==0)){
+            if((minmean == 1) | (sum(res_nlvsum_rmseCV_Ysel$mean[1:minmean]<=threshmean)==0)){
               kchoix <- 1
             }else{
-              kchoix <- max(res_nlvsum_rmseCV_Ysel[which(res_nlvsum_rmseCV_Ysel$mean[1:minmean]>=threshmean),"index"])
+              kchoix <- min(res_nlvsum_rmseCV_Ysel[which(res_nlvsum_rmseCV_Ysel$mean[1:minmean]<=threshmean),"index"])
             }
           }else{
             kchoix <- 1
@@ -437,7 +397,6 @@
       
       # optim combination
       optimcombi <- as.vector(unlist(lvcombi[[m]][kchoix,]))
-      #names(optimcombi) <- paste0("Xlist",1:m)
       nlvlist[[m]] <- lvcombi[[m]][kchoix,m]
     }# end loop on m
  
@@ -447,9 +406,7 @@
 
     # outputs
     rts <- list(lvcombi=lvcombi,
-                #optimCombiLine=kchoix, 
                 optimcombi=unlist(lvcombi[[nXblocks]][kchoix,,drop=FALSE]), 
-                #optimExplVarCV=res_ExplVarCV[[nXblocks]][kchoix,,drop=FALSE], 
                 rmseCV_byY=res_rmseCV_byY, 
                 ExplVarCV_byY=res_ExplVarCV_byY, 
                 rmseCV=res_rmseCV, 
